@@ -9,12 +9,16 @@ use Mojo::Parameters;
 use Mojo::Util 'encode';
 use Selenium::Remote::WDKeys ();
 
-use constant DEBUG => $ENV{MOJO_SELENIUM_DEBUG} || 0;
+use constant DEBUG         => $ENV{MOJO_SELENIUM_DEBUG}         || 0;
+use constant WAIT_INTERVAL => $ENV{MOJO_SELENIUM_WAIT_INTERVAL} || 0.5;
+use constant WAIT_TIMEOUT  => $ENV{MOJO_SELENIUM_WAIT_TIMEOUT}  || 60;
 
 $ENV{TEST_SELENIUM} //= '0';
 $ENV{MOJO_SELENIUM_BASE_URL} ||= $ENV{TEST_SELENIUM} =~ /^http/ ? $ENV{TEST_SELENIUM} : '';
 
-our $VERSION = '0.15';
+sub S { Mojo::JSON::encode_json($_[0]) }
+
+our $VERSION = '0.16';
 
 my $SCRIPT_NAME = File::Basename::basename($0);
 my $SCREENSHOT  = 1;
@@ -73,7 +77,7 @@ sub active_element_is {
   my $el     = $self->_find_element($selector);
   my $same   = $active && $el ? $driver->compare_elements($active, $el) : 0;
 
-  return $self->_test('ok', $same, _desc($desc, "active element is $selector"));
+  return $self->_test('ok', $same, _desc($desc, "active element is @{[S $selector]}"));
 }
 
 sub capture_screenshot {
@@ -97,7 +101,7 @@ sub click_ok {
     $err = '' if $el->click;
   }
 
-  return $self->_test('ok', !$err, _desc("click on $selector $err"));
+  return $self->_test('ok', !$err, _desc("click on @{[S $selector]} $err"));
 }
 
 sub current_url_is {
@@ -117,14 +121,18 @@ sub current_url_like {
 sub element_is_displayed {
   my ($self, $selector, $desc) = @_;
   my $el = $self->_find_element($selector);
-  return $self->_test('ok', ($el && $el->is_displayed),
-    _desc($desc, "element $selector is displayed"));
+  return $self->_test(
+    'ok',
+    ($el && $el->is_displayed),
+    _desc($desc, "element @{[S $selector]} is displayed")
+  );
 }
 
 sub element_is_hidden {
   my ($self, $selector, $desc) = @_;
   my $el = $self->_find_element($selector);
-  return $self->_test('ok', ($el && $el->is_hidden), _desc($desc, "element $selector is hidden"));
+  return $self->_test('ok', ($el && $el->is_hidden),
+    _desc($desc, "element @{[S $selector]} is hidden"));
 }
 
 sub go_back    { $_[0]->_proxy('go_back');    $_[0] }
@@ -146,18 +154,18 @@ sub live_element_count_is {
   my ($self, $selector, $count, $desc) = @_;
   my $els = $self->_proxy(find_elements => $selector);
   return $self->_test('is', int(@$els), $count,
-    _desc($desc, qq{element count for selector "$selector"}));
+    _desc($desc, "element count for selector @{[S $selector]}"));
 }
 
 sub live_element_exists {
   my ($self, $selector, $desc) = @_;
-  $desc = _desc($desc, qq{element for selector "$selector" exists});
+  $desc = _desc($desc, "element for selector @{[S $selector]} exists");
   return $self->_test('ok', $self->_find_element($selector), $desc);
 }
 
 sub live_element_exists_not {
   my ($self, $selector, $desc) = @_;
-  $desc = _desc($desc, qq{no element for selector "$selector"});
+  $desc = _desc($desc, "no element for selector @{[S $selector]}");
   return $self->_test('ok', !$self->_find_element($selector), $desc);
 }
 
@@ -165,7 +173,7 @@ sub live_text_is {
   my ($self, $selector, $value, $desc) = @_;
   return $self->_test(
     'is', $self->_element_data(get_text => $selector),
-    $value, _desc($desc, qq{exact text for selector "$selector"})
+    $value, _desc($desc, "exact text for selector @{[S $selector]}")
   );
 }
 
@@ -173,7 +181,7 @@ sub live_text_like {
   my ($self, $selector, $regex, $desc) = @_;
   return $self->_test(
     'like', $self->_element_data(get_text => $selector),
-    $regex, _desc($desc, qq{similar text for selector "$selector"})
+    $regex, _desc($desc, "similar text for selector @{[S $selector]}")
   );
 }
 
@@ -181,7 +189,7 @@ sub live_value_is {
   my ($self, $selector, $value, $desc) = @_;
   return $self->_test(
     'is', $self->_element_data(get_value => $selector),
-    $value, _desc($desc, qq{exact value for selector "$selector"})
+    $value, _desc($desc, "exact value for selector @{[S $selector]}")
   );
 }
 
@@ -189,7 +197,7 @@ sub live_value_like {
   my ($self, $selector, $regex, $desc) = @_;
   return $self->_test(
     'like', $self->_element_data(get_value => $selector),
-    $regex, _desc($desc, qq{similar value for selector "$selector"})
+    $regex, _desc($desc, "similar value for selector @{[S $selector]}")
   );
 }
 
@@ -252,7 +260,7 @@ sub send_keys_ok {
   }
 
   $keys = Mojo::Util::url_escape(join '', @$keys);
-  return $self->_test('ok', $el, _desc($desc, "keys ($keys) sent to $selector"));
+  return $self->_test('ok', $el, _desc($desc, "keys ($keys) sent to @{[S $selector]}"));
 }
 
 sub set_window_size {
@@ -277,7 +285,7 @@ sub submit_ok {
   my ($self, $selector, $desc) = @_;
   my $el = $self->_find_element($selector);
   $el->submit if $el;
-  return $self->_test('ok', $el, _desc($desc, "click on $selector"));
+  return $self->_test('ok', $el, _desc($desc, "click on @{[S $selector]}"));
 }
 
 sub toggle_checked_ok {
@@ -297,61 +305,67 @@ sub toggle_checked_ok {
     }
   }
 
-  return $self->_test('ok', $el, _desc("click on $selector"));
+  return $self->_test('ok', $el, _desc("click on @{[S $selector]}"));
 }
 
 sub wait_for {
-  my ($self, $arg, $desc) = @_;
+  my $self = shift;
+  my $sel  = shift;
+  my $args = ref $_[0] eq 'HASH' ? shift : {};
+  my $desc = shift || "waited for element $sel";
   my @checks;
 
-  return $self->wait_until(sub {0}, {skip => 1, timeout => $arg})
-    if Scalar::Util::looks_like_number($arg);
+  if (Scalar::Util::looks_like_number($sel)) {
+    $self->ua->ioloop->timer($sel => sub { shift->stop });
+    $self->ua->ioloop->start;
+    return $self;
+  }
 
+  push @checks, 'is_displayed' if $sel =~ s!:visible\b!!;
+  push @checks, 'is_enabled'   if $sel =~ s!:enabled\b!!;
+  push @checks, 'is_hidden'    if $sel =~ s!:hidden\b!!;
+  push @checks, 'is_selected'  if $sel =~ s!:selected\b!!;
 
-  $desc ||= "waited for element $arg";
-  push @checks, 'is_displayed' if $arg =~ s!:visible\b!!;
-  push @checks, 'is_enabled'   if $arg =~ s!:enabled\b!!;
-  push @checks, 'is_hidden'    if $arg =~ s!:hidden\b!!;
-  push @checks, 'is_selected'  if $arg =~ s!:selected\b!!;
+  my $driver                = $self->driver;
+  my $prev_implicit_timeout = $driver->get_timeouts->{implicit};
+  $driver->set_timeout(implicit => $args->{timeout} || WAIT_TIMEOUT);
 
-  return $self->wait_until(
+  my $ok;
+  $self->wait_until(
     sub {
-      my $e = $_->find_element($arg);
-      return $e && @checks == grep { $e->$_ } @checks;
+      my $e = $_->find_element($sel);
+      return $ok = $e && @checks == grep { $e->$_ } @checks;
     },
-    {desc => $desc},
+    {%$args, skip => 1},
   );
+
+  $driver->set_timeout(implicit => $prev_implicit_timeout);
+
+  return $self->_test('ok', $ok, _desc($desc));
 }
 
 sub wait_until {
   my ($self, $cb, $args) = @_;
   my $ioloop = $self->ua->ioloop;
-  my $t0     = time;
-  my ($ok, @tid);
 
-  $args->{timeout}  ||= $ENV{MOJO_SELENIUM_WAIT_TIMEOUT}  || 60;
-  $args->{interval} ||= $ENV{MOJO_SELENIUM_WAIT_INTERVAL} || 0.5;
+  my $err;
+  my @tid = (
+    $ioloop->timer($args->{timeout} || WAIT_TIMEOUT, sub { $err = 'Timeout' }),
+    $ioloop->recurring(
+      $args->{interval} || WAIT_INTERVAL,
+      sub {
+        return shift->stop if $err || eval { local $_ = $self->driver; $self->$cb($args) };
+        Test::More::diag("[Selenium] wait_until: $@") if $@ and ($args->{debug} or DEBUG);
+      }
+    ),
+  );
 
-  $ioloop->delay(
-    sub {
-      my $next = shift->begin;
-      push @tid, $ioloop->timer($args->{timeout}, $next);
-      push @tid, $ioloop->recurring(
-        $args->{interval},
-        sub {
-          $next->(1, 1) if eval { local $_ = $self->driver; $self->$cb($args) };
-          Test::More::diag("[Selenium] wait_until: $@") if $@ and ($args->{debug} or DEBUG);
-        }
-      );
-    },
-    sub {
-      $ok = $_[1];
-      $ioloop->remove($_) for @tid;
-    },
-  )->wait;
+  my $t0 = time;
+  $ioloop->start;
+  $ioloop->remove($_) for @tid;
 
   return $self if $args->{skip};
-  return $self->_test('ok', $ok, _desc($args->{desc} || "waited for @{[time - $t0]}s"));
+  return $self->_test('ok', !$err, _desc($args->{desc} || "waited for @{[time - $t0]}s"));
 }
 
 sub window_size_is {
@@ -830,12 +844,13 @@ javascript.
 
   $self = $self->wait_for(0.2);
   $self = $self->wait_for('[name="agree"]', "test description");
-  $self = $self->wait_for('[name="agree"]:enabled');
+  $self = $self->wait_for('[name="agree"]:enabled', {interval => 1.5, timeout => 10});
   $self = $self->wait_for('[name="agree"]:selected');
   $self = $self->wait_for('[href="/"]:visible');
   $self = $self->wait_for('[href="/hidden"]:hidden');
+  $self = $self->wait_for('[name=checkbox]:checked');
 
-Simpler version of L</wait_for> for the most common use cases:
+Simpler version of L</wait_until> for the most common use cases:
 
 =over 2
 
